<?php
namespace Models;

// Модель для работы с Банковским терминалом
class BankTerminal extends \DB\SQL\Mapper{
	public $db, $f3, $sets, $bt;

	public function __construct($db, $f3) {
		parent::__construct($db, 'settings'); // подключаемся к таблице с настройками
		$this->f3=$f3;
		$this->db=$db;
		$this->sets=$this->get_sets();
		if($this->sets['bt_active'] && $this->sets['bt_driver']!='' && file_exists($this->sets['bt_driver'].'/server.php')) {
			$class=explode('/', $this->sets['bt_driver']);
			$class=array_reverse($class);
			$model='\Models\\'.$class[0];
			if(!class_exists($model, false)) {
				require_once($this->sets['bt_driver'].'/server.php');
			}
			$file='/server.config';
			$config=parse_ini_file($this->sets['bt_driver'].$file);
			$this->bt=new $model($f3, $this->sets, $config);
		}
		else {
			$this->sets['bt_active']=0;
		}
	}

	function get_sets() {
		$sets=array(
			'bt_active'=>0,
			'bt_driver'=>''
		);
		$this->reset();
		$this->load(array('name IN (?, ?)', array('bt_active', 'bt_driver')));
		if($this->dry()) {
			return $sets;
		}
		do {
			$sets[$this->name]=$this->value;
		}
		while($this->skip());
		if(!$sets['bt_driver']) {
			$sets['bt_active']=0;
		}
		return $sets;
	}

	// Список драйверов
	function get_drivers() {
		$drivers=array();
		$bankterminal_dir=$this->f3->get('bankterminal_dir');
		$dirs=scandir($bankterminal_dir); // Ищем все драйвера
		if($dirs) {
			foreach ($dirs as $d) {
				$folder=$bankterminal_dir.$d;
				$file=$folder.'/server.config';
				if(is_dir($folder) && file_exists($file)) {
					$config=parse_ini_file($file);
					if($config) {
						$drivers[$folder]=array(
							'title'=>$config['ServerName'],
							'path'=>$folder
						);
					}
				}
			}
		}

		return $drivers;
	}

	// Сохраняем настройки
	function save_sets($data) {
		$sql="INSERT OR REPLACE INTO `settings` (`name`, `value`) VALUES ";
		$val=array();
		foreach ($data as $key => $value) {
			if(!empty($val)) {
				$sql.=", ";
			}
			$sql.="(?, ?)";
			$val[]=$key;
			$val[]=$value;
		}
		$res=$this->db->exec($sql, $val);
		if($res===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}

	function ticket($type, $sum) {
		if(!$this->sets['bt_active']) {
			return array('success'=>true);
		}
		return $this->bt->ticket($type, $sum);
	}

	function z() {
		if(!$this->sets['bt_active']) {
			return array('success'=>true);
		}
		return $this->bt->z();
	}

	function x() {
		if(!$this->sets['bt_active']) {
			return array('success'=>true);
		}
		return $this->bt->x();
	}

	function cashermenu() {
		if(!$this->sets['bt_active']) {
			return array('success'=>true);
		}
		$res=$this->bt->cashermenu();
		if($res['success']) {
			$res['close_all_window']=true;
		}
		return $res;
	}

	function adminmenu() {
		if(!$this->sets['bt_active']) {
			return array('success'=>true);
		}
		$res=$this->bt->adminmenu();
		if($res['success']) {
			$res['close_all_window']=true;
		}
		return $res;
	}
}